@@ -18,6 +18,7 @@
  * 13.2.2013, added WD option --Sampo
  * 21.6.2013, added SOAP WSP capability --Sampo
  * 22.6.2013, created, based on mod_auth_saml.c and zxidwspcgi.c --Sampo
+ * 10.11.2013, many bugs fixed, much improved --Sampo
  *
  * See also: zxidwspcgi.c, mod_auth_saml.c
  */
@@ -40,38 +41,35 @@
 void send_error_and_exit(int s, char* title, char* extra_header, char* text);
 ssize_t my_read(char* buf, size_t size);
 ssize_t my_write(char* buf, size_t size);
-void add_to_buf( char** bufP, size_t* bufsizeP, size_t* buflenP, char* str, size_t len );
-void add_to_response( char* str, size_t len );
-void add_to_request( char* str, size_t len );
+void add_to_buf(char** bufP, size_t* bufsizeP, size_t* buflenP, char* str, size_t len);
+void add_to_request(char* str, size_t len);
+void add_headers(int s, char* title, char* extra_header, char* me, char* mt, off_t b, time_t mod);
+void add_to_response(char* str, size_t len);
+void send_response(void);
+extern char* remoteuser;
+extern char* path;
 extern char* request;
 extern size_t request_size, request_len, request_idx;
 extern size_t content_length;
 extern int zxid_is_wsp;               /* Flag to trigger WSP response decoration. */
+extern zxid_ses* zxid_session;
 
 /*() Convert session attribute pool into mini_httpd CGI execution environment.
  *
  * OUTMAP will be applied to decide which attributes to pass to the environment
  * and to rename them.
  *
- * This is considered internal function to mod_auth_saml, called by chkuid().
+ * This is considered internal function to mini_httpd_zxid, called by make_envp() in do_cgi().
  * You should not call this directly, unless you know what you are doing. */
 
-/* Called by:  chkuid */
+/* Called by:  make_envp */
 int zxid_pool2env(zxid_conf* cf, zxid_ses* ses, char** envp, int envn, int max_envn, const char* uri_path, const char* qs)
 {
-  char* rs_qs;
   char* name;
-  char* rs = 0;
-  char* setcookie = 0;
-  char* setptmcookie = 0;
-  char* cookie = 0;
-  char* idpnid = 0;
   struct zxid_map* map;
   struct zxid_attr* at;
   struct zxid_attr* av;
   
-  /* Length computation pass */
-
   for (at = ses->at; at; at = at->n) {
     DD("HERE name(%s)", at->name);
     map = zxid_find_map(cf->outmap, at->name);
@@ -98,7 +96,7 @@ int zxid_pool2env(zxid_conf* cf, zxid_ses* ses, char** envp, int envn, int max_e
 					cf->mod_saml_attr_prefix, name, av->map_val->s);
       }
     } else {
-      if ((zx_debug & ZX_DEBUG_MASK)>1)
+      if ((errmac_debug & ERRMAC_DEBUG_MASK)>1)
 	D("ATTR(%s)=VAL(%s)", at->name, STRNULLCHKNULL(at->val));
       else
 	D("ATTR(%s)=VAL(%.*s)", at->name, at->val?(int)MIN(35,strlen(at->val)):6, at->val?at->val:"(null)");
@@ -114,61 +112,13 @@ int zxid_pool2env(zxid_conf* cf, zxid_ses* ses, char** envp, int envn, int max_e
 					cf->mod_saml_attr_prefix, at->name, av->val);
       }
     }
-    if (     !strcmp(at->name, "rs"))           rs = at->val;      /* Capture special */
-    else if (!strcmp(at->name, "idpnid"))       idpnid = at->val;
-    else if (!strcmp(at->name, "setcookie"))    setcookie = at->val;
-    else if (!strcmp(at->name, "setptmcookie")) setptmcookie = at->val;
-    else if (!strcmp(at->name, "cookie"))       cookie = at->val;
-  }
-
-  if (rs && rs[0] && rs[0] != '-') {
-    /* N.B. RelayState was set by chkuid() "some other page" section by setting cgi.rs
-     * to deflated and safe base64 encoded value which was then sent to IdP as RelayState.
-     * It then came back from IdP and was decoded as one of the SSO attributes.
-     * The decoding is controlled by <<tt: rsrc$rs$unsb64-inf$$ >>  rule in OUTMAP. */
-#if 1
-    rs = zxid_unbase64_inflate(cf->ctx, -2, rs, 0);
-    if (!rs) {
-      ERR("Bad relaystate. Error in inflate. %d", 0);
-      send_error_and_exit(400, "Bad Request", "", "Error in relaystate inflate");
-    }
-#endif
-    rs_qs = strchr(rs, '?');
-    if (rs_qs
-	?(memcmp(uri_path, rs, rs_qs-rs)||strcmp(qs?qs:"",rs_qs+1))
-	:strcmp(uri_path, rs)) {  /* Different, need external or internal redirect */
-      D("redirect(%s) redir_to_content=%d", rs, cf->redir_to_content);
-      //r->uri = apr_pstrdup(r->pool, val);
-      if (cf->redir_to_content) {
-	send_error_and_exit(302, "Found", rs, "SAML Redirect" );
-      } else {
-	/* *** for this to be effective, it should be done much earlier, already where
-	 *     filter is called. By this time the file global has already been computed. */
-	uri_path = rs;
-      }
+    if (!strcmp(at->name, "idpnid") && at->val && at->val[0] != '-') {
+      D("REMOTE_USER(%s)", at->val);
+      remoteuser = at->val;
     }
   }
   
-#if 0
-  /* *** figure out how to influence mini_httpd response headers */
-  if (setcookie && setcookie[0] && setcookie[0] != '-') {
-    D("Set-Cookie(%s)", setcookie);
-    apr_table_addn(r->headers_out, "Set-Cookie", setcookie);
-    apr_table_addn(r->err_headers_out, "Set-Cookie", setcookie);  /* Only way to get redir to set header */
-  }
-  if (setptmcookie && setptmcookie[0] && setptmcookie[0] != '-') {
-    D("PTM Set-Cookie(%s)", setptmcookie);
-    apr_table_addn(r->headers_out, "Set-Cookie", setptmcookie);
-    apr_table_addn(r->err_headers_out, "Set-Cookie", setptmcookie);  /* Only way to get redir to set header */
-  }
-#endif
-  if (idpnid && idpnid[0] != '-') {
-    D("REMOTE_USER(%s)", idpnid);
-    if (envn >= max_envn) goto enverr;
-    envp[envn++] = zx_alloc_sprintf(cf->ctx, 0, "REMOTE_USER=%s", idpnid);
-  }
-  
-  D("SSO OK uri(%s) rs(%s)", uri_path, STRNULLCHKQ(rs));
+  D("CGI SSO OK uri(%s)", uri_path);
   return envn;
  enverr:
   ERR("Statically allocated CGI environment array too small. max_envn=%d", max_envn);
@@ -181,26 +131,34 @@ int zxid_pool2env(zxid_conf* cf, zxid_ses* ses, char** envp, int envn, int max_e
  * It works by accessing certain request related global variables from mini_httpd.
  * You should not call this directly, unless you know what you are doing. */
 
-/* Called by: */
+/* Called by:  zxid_mini_httpd_sso, zxid_mini_httpd_wsp */
 static char* zxid_mini_httpd_read_post(zxid_conf* cf)
 {
   char* res;
 
   for (;;) {
-    char buf[10000];
+    char buf[32*1024];
     int already_read = request_len-request_idx;
-    int len = my_read(buf, MIN(sizeof(buf), content_length - already_read));
+    int len = MIN(sizeof(buf), content_length - already_read);
+    D("Read post already_read=%d/%d buf_siz=%d len=%d", already_read, (int)content_length, (int)sizeof(buf), len);
+    DD("uri(%s)=%p buf=%p request(%.*s)=%p request_size=%d request_len=%d", path, path, buf, (int)request_size, request, request, (int)request_size, (int)request_len);
+    if (!len)
+      break;  /* nothing further to read */
+    len = my_read(buf, len);
     if (len < 0 && ONE_OF_2(errno, EINTR, EAGAIN))
       continue;
     if (len <= 0)
       break;
+    DD("uri(%s)=%p buf=%p request(%.*s)=%p request_size=%d request_len=%d", path, path, buf, (int)request_size, request, request, (int)request_size, (int)request_len);
     add_to_request(buf, len);
+    DD("uri(%s)=%p buf=%p request(%.*s)=%p request_size=%d request_len=%d", path, path, buf, (int)request_size, request, request, (int)request_size, (int)request_len);
   }
   res = request + request_idx;
-  if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("POST(%s)", res);
+  if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("POST(%s)", res);
   return res;
 }
 
+/* Called by:  zxid_mini_httpd_filter */
 zxid_ses* zxid_mini_httpd_wsp(zxid_conf* cf, const char* method, const char* uri_path, const char* qs)
 {  
   char* res;
@@ -226,11 +184,13 @@ zxid_ses* zxid_mini_httpd_wsp(zxid_conf* cf, const char* method, const char* uri
 /*() Handle the WSP case of cgi_interpose_output(). Read in entire response,
  * apply decoration, and send it on its way. */
 
+/* Called by:  cgi_interpose_output */
 void zxid_mini_httpd_wsp_response(zxid_conf* cf, zxid_ses* ses, int rfd, char** response, size_t* response_size, size_t* response_len, int br_ix)
 {  
   struct zx_str* res;
   
-  D("DECOR START response_size=%d response_len=%d br_ix=%d response(%.*s)", *response_size, *response_len, br_ix, *response_len, *response);
+  D_INDENT("wsp_resp");
+  D("DECOR START response_size=%d response_len=%d br_ix=%d response(%.*s)", (int)*response_size, (int)*response_len, br_ix, (int)*response_len, *response);
 
   /* Read until EOF */
   for (;;) {
@@ -245,7 +205,7 @@ void zxid_mini_httpd_wsp_response(zxid_conf* cf, zxid_ses* ses, int rfd, char**
     add_to_buf(response, response_size, response_len, buf, len);
   }
   
-  D("DECOR2 response_size=%d response_len=%d br_ix=%d response(%.*s)", *response_size, *response_len, br_ix, *response_len, *response);
+  D("DECOR2 response_size=%d response_len=%d br_ix=%d response(%.*s)", (int)*response_size, (int)*response_len, br_ix, (int)*response_len, *response);
 
   /* Write the saved headers (and any beginning of payload). */
   if ((*response)[br_ix] == '\015') ++br_ix;
@@ -253,52 +213,58 @@ void zxid_mini_httpd_wsp_response(zxid_conf* cf, zxid_ses* ses, int rfd, char**
   if ((*response)[br_ix] == '\015') ++br_ix;
   if ((*response)[br_ix] == '\012') ++br_ix;
 
-  D("DECOR3 response_len=%d br_ix=%d header(%.*s)", *response_len, br_ix, br_ix, *response);
+  D("DECOR3 response_len=%d br_ix=%d header(%.*s)", (int)*response_len, br_ix, br_ix, *response);
   (void) my_write(*response, br_ix);
 
   res = zxid_wsp_decorate(cf, ses, 0, *response+br_ix);
   (void) my_write(res->s, res->len);
+  D_DEDENT("wsp_resp");
 }
 
-/* 0x6000 outf QS + JSON = no output
+/* 0x6000 outf QS + JSON = no output on successful sso, the attrubutes are in session
  * 0x1000 debug
  * 0x0e00 11 + 10 = Generate all HTML + Mgmt w/headers as string
  * 0x00a0 10 + 10 = Login w/headers as string + Meta w/headers as string
  * 0x0008 10 + 00 = SOAP w/headers as string + no auto redir, no exit(2) */
 #define AUTO_FLAGS 0x6ea8
 
+/* Called by:  zxid_mini_httpd_filter */
 zxid_ses* zxid_mini_httpd_sso(zxid_conf* cf, const char* method, const char* uri_path, const char* qs, const char* cookie_hdr)
 {  
-  int ret, len, uri_len, url_len, args_len;
+  int ret, len, uri_len, url_len, qs_len;
+  char* local_url;
   char* p;
   char* res;
+  char* mt;
   //const char* set_cookie_hdr;
   //const char* cur_auth;
   zxid_ses* ses = zxid_alloc_ses(cf);
   zxid_cgi cgi;
   ZERO(&cgi, sizeof(zxid_cgi));
+  cgi.uri_path = (char*)uri_path;
+  cgi.qs = (char*)qs;
 
-  /* Probe for Session ID in cookie. Also propagate the cookie to subrequests. */
+  /* Probe for Session ID in cookie. */
 
   if (cf->ses_cookie_name && *cf->ses_cookie_name) {
     if (cookie_hdr) {
       D("found cookie(%s) 3", STRNULLCHK(cookie_hdr));
       zxid_get_sid_from_cookie(cf, &cgi, cookie_hdr);
-      /*set_cookie_hdr = apr_table_get(r->headers_in, "Set-Cookie");  // for subreqs */
     }
   }
-
+  
   /* Redirect hack: deal with externally imposed ACS url that does not follow zxid convention. */
   
-  args_len = qs?strlen(qs):0;
+  qs_len = qs?strlen(qs):0;
   if (cf->redirect_hack_imposed_url && !strcmp(uri_path, cf->redirect_hack_imposed_url)) {
     D("Redirect hack: mapping(%s) imposed to zxid(%s)", uri_path, cf->redirect_hack_zxid_url);
     uri_path = cf->redirect_hack_zxid_url;
+    cgi.uri_path = (char*)uri_path;
     if (cf->redirect_hack_zxid_qs && *cf->redirect_hack_zxid_qs) {
-      if (args_len) {
+      if (qs_len) {
 	/* concatenate redirect_hack_zxid_qs with existing qs */
 	len = strlen(cf->redirect_hack_zxid_qs);
-	p = ZX_ALLOC(cf->ctx, len+1+args_len+1);
+	p = ZX_ALLOC(cf->ctx, len+1+qs_len+1);
 	strcpy(p, cf->redirect_hack_zxid_qs);
 	p[len] = '&';
 	strcpy(p+len+1, qs);
@@ -306,39 +272,46 @@ zxid_ses* zxid_mini_httpd_sso(zxid_conf* cf, const char* method, const char* uri
       } else {
 	qs = cf->redirect_hack_zxid_qs;
       }
-      args_len = strlen(qs);
+      qs_len = strlen(qs);
     }
     D("After hack uri(%s) args(%s)", STRNULLCHK(uri_path), STRNULLCHK(qs));
   }
   
-  DD("HERE1 args_len=%d cgi=%p k(%s) args(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(qs));
-  if (args_len) {
+  D("HERE1 qs_len=%d cgi=%p k(%s) qs(%s)", qs_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(qs));
+  if (qs_len) {
     /* leak the dup str: the cgi structure will take references to this and change &s to nuls */
     p = zx_dup_cstr(cf->ctx, qs);
     zxid_parse_cgi(cf, &cgi, p);
-    DD("HERE2 args_len=%d cgi=%p k(%s) args(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), STRNULLCHKNULL(qs));
+    D("HERE2 cgi=%p k(%s)", &cgi, STRNULLCHKNULL(cgi.skin));
   }
   /* Check if we are supposed to enter zxid due to URL suffix - to
    * process protocol messages rather than ordinary pages. To do this
    * correctly we need to ignore the query string part. We are looking
-   * here at exact match, like /protected/saml, rather than any of
+   * here at an exact match, like /protected/saml, rather than any of
    * the other documents under /protected/ (which are handled in the
    * else clause). Both then and else -clause URLs are defined as requiring
    * SSO by virtue of the web server configuration (SSO_PAT in mini_httpd_zxid). */
 
   uri_len = strlen(uri_path);
-  url_len = strlen(cf->url);
-  for (p = cf->url + url_len - 1; p > cf->url; --p)
+  for (local_url = cf->url; *local_url && *local_url != ':' && *local_url != '/'; ++local_url);
+  if (local_url[0] == ':' && local_url[1] == '/' && local_url[2] == '/') {
+    for (local_url += 3; *local_url && *local_url != '/'; ++local_url);
+  }
+  
+  url_len = strlen(local_url);
+  for (p = local_url + url_len - 1; p > local_url; --p)
     if (*p == '?')
       break;
-  if (p == cf->url)
-    p = cf->url + url_len;
-  
-  if (url_len >= uri_len && !memcmp(p - uri_len, uri_path, uri_len)) {  /* Suffix match */
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("matched uri(%s) cf->url(%s) qs(%s) rs(%s) op(%c)", uri_path, cf->url, STRNULLCHKNULL(qs), STRNULLCHKNULL(cgi.rs), cgi.op);
+  if (p == local_url)
+    p = local_url + url_len;
+  url_len = p-local_url;
+
+  if (url_len == uri_len && !memcmp(local_url, uri_path, uri_len)) {  /* Exact match */
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("matched uri(%s)=%p cf->url(%s) qs(%s) rs(%s) op(%c)", uri_path, uri_path, cf->url, STRNULLCHKNULL(qs), STRNULLCHKNULL(cgi.rs), cgi.op);
     if (*method == 'P') {
       res = zxid_mini_httpd_read_post(cf);   /* Will print some debug output */  // ***
       if (res) {
+	DD("uri(%s)=%p", uri_path, uri_path);
 	if (cgi.op == 'S') {
 	  ret = zxid_sp_soap_parse(cf, &cgi, ses, strlen(res), res);
 	  D("POST soap parse returned %d", ret);
@@ -363,41 +336,36 @@ zxid_ses* zxid_mini_httpd_sso(zxid_conf* cf, const char* method, const char* uri
 	} else {
 	  zxid_parse_cgi(cf, &cgi, res);
 	  D("POST CGI parsed. rs(%s)", STRNULLCHKQ(cgi.rs));
+	  DD("uri(%s)=%p", uri_path, uri_path);
 	}
       }
     }
+    D("HERE2.1 urls_len=%d local_url(%.*s) url(%s)", url_len, url_len, local_url, cf->url);
     if (ONE_OF_2(cgi.op, 'L', 'A')) /* SSO (Login, Artifact) activity overrides current session. */
       goto step_up;
     if (!cgi.sid || !zxid_get_ses(cf, ses, cgi.sid)) {
-      D("No session(%s) active op(%c)", STRNULLCHK(cgi.sid), cgi.op);
+      D("No session(%s) active op(%c) uri(%s)=%p", STRNULLCHK(cgi.sid), cgi.op, uri_path,uri_path);
     } else {
+      D("HERE2.2 %d",0);
       res = zxid_simple_ses_active_cf(cf, &cgi, ses, 0, AUTO_FLAGS);
       if (res)
 	goto process_zxid_simple_outcome;
     }
-    /* not logged in, fall thru */
+    /* not logged in, fall thru to step_up */
   } else {
     /* Some other page. Just check for session. */
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("other page uri(%s) qs(%s) cf->url(%s) uri_len=%d url_len=%d", uri_path, STRNULLCHKNULL(qs), cf->url, uri_len, url_len);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("other page uri(%s) qs(%s) cf->url(%s) uri_len=%d url_len=%d", uri_path, STRNULLCHKNULL(qs), cf->url, uri_len, url_len);
     if (qs && qs[0] == 'l') {
       D("Detect login(%s)", qs);
     } else
       cgi.op = 'E';   /* Trigger IdP selection screen */
-    p = ZX_ALLOC(cf->ctx, uri_len+1+args_len+1);
+    p = ZX_ALLOC(cf->ctx, uri_len+1+qs_len+1);
     strcpy(p, uri_path);
-    if (args_len) {
+    if (qs_len) {
       p[uri_len] = '?';
       strcpy(p+uri_len+1, qs);
     }
-    DD("HERE3 args_len=%d cgi=%p k(%s) uri(%s) args(%s) rs(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), uri_path, STRNULLCHKNULL(qs), p);
-    /* cgi.rs will be copied to ses->rs and from there in ab_pep to resource-id.
-     * We compress and safe_base64 encode it to protect any URL special characters.
-     * *** seems that at this point the p is not just rs, but the entire local URL --Sampo */
-    cgi.rs = zxid_deflate_safe_b64_raw(cf->ctx, -2, p);
-    if (cf->defaultqs && cf->defaultqs[0]) {
-      if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("DEFAULTQS(%s)", cf->defaultqs);
-      zxid_parse_cgi(cf, &cgi, cf->defaultqs);
-    }
+    D("HERE3 qs_len=%d cgi=%p k(%s) uri(%s) qs(%s) rs(%s)", qs_len, &cgi, STRNULLCHKNULL(cgi.skin), uri_path, STRNULLCHKNULL(qs), p);
     if (cgi.sid && cgi.sid[0] && zxid_get_ses(cf, ses, cgi.sid)) {
       res = zxid_simple_ses_active_cf(cf, &cgi, ses, 0, AUTO_FLAGS);
       if (res)
@@ -405,51 +373,52 @@ zxid_ses* zxid_mini_httpd_sso(zxid_conf* cf, const char* method, const char* uri
     } else {
       D("No session(%s) active op(%c)", STRNULLCHK(cgi.sid), cgi.op);
     }
-    D("other page: no_ses uri(%s) templ(%s) tf(%s) k(%s) cgi=%p", uri_path, STRNULLCHKNULL(cgi.templ), STRNULLCHKNULL(cf->idp_sel_templ_file), cgi.skin, &cgi);
+    D("other page: no_ses uri(%s) templ(%s) tf(%s) k(%s) cgi=%p rs(%s)", uri_path, STRNULLCHKNULL(cgi.templ), STRNULLCHKNULL(cf->idp_sel_templ_file), cgi.skin, &cgi, cgi.rs);
   }
 step_up:
+  DD("before uri(%s)=%p", uri_path, uri_path);
   res = zxid_simple_no_ses_cf(cf, &cgi, ses, 0, AUTO_FLAGS);
+  DD("after uri(%s)", uri_path);
 
 process_zxid_simple_outcome:
   if (cookie_hdr && cookie_hdr[0]) {
-    D("Passing cookie(%s) to environment", cookie_hdr);
+    D("Passing previous cookie(%s) to environment", cookie_hdr);
     zxid_add_attr_to_ses(cf, ses, "cookie", zx_dup_str(cf->ctx, cookie_hdr));
   }
-
+  D("res(%s) uri(%s)",res,uri_path);
   switch (res[0]) {
   case 'L':
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("REDIR(%s)", res);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("REDIR(%s)", res);
+    zxid_session = ses; /* Set the session so that the mini_httpd add_headers() can set cookies */
     send_error_and_exit(302, "Found", res, "SAML Redirect");
   case 'C':
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("CONTENT(%s)", res);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("CONTENT(%s)", res);
     res += 14;  /* skip "Content-Type:" (14 chars) */
     DD("RES(%s)", res);
     p = strchr(res, '\r');
     *p = 0;
-    //mt = res;
+    mt = res;
     DD("CONTENT-TYPE(%s)", res);
     res = p+2 + 16;  /* skip "Content-Length:" (16 chars) */
     sscanf(res, "%d", &len);
     res = strchr(res, '\r') + 4; /* skip CRFL pair before body */
     DD("CONTENT-LENGTH(%d)", len);
-    //add_headers(200, "OK", "", "", mt, len, (time_t)-1);
-    // *** add_to_response( res, buflen );
-    // *** send_response();
-#ifdef USE_SSL
-    SSL_free( ssl );
-#endif /* USE_SSL */
-    exit(0);
+    zxid_session = ses; /* Set the session so that the mini_httpd add_headers() can set cookies */
+    add_headers(200, "OK", "", "", mt?mt:"text/html; charset=%s", len, (time_t)-1);
+    add_to_response(res, len);
+    send_response();
+    exit(0);  /* This function is called in mini_httpd handle_request() subrprocess. */
   case 'z':
     INFO("User not authorized %d", 0);
     send_error_and_exit(403, "Forbidden", "", "Authorization denied.");
   case 0: /* Logged in case */
-    D("SSO OK pre uri(%s)", uri_path);
-    //ret = pool2apache(cf, r, ses.at); // *** done in docgi()
-    return 0;
+    D("SSO OK uri(%s)", uri_path);
+    /*ret = pool2apache(cf, r, ses.at); // will be done in do_cgi() */
+    break;
 #if 0
   case 'd': /* Logged in case */
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("SSO OK LDIF(%s)", res);
-    D("SSO OK pre uri(%s)", uri_path);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("SSO OK LDIF(%s)", res);
+    D("SSO OK uri(%s)", uri_path);
     ret = ldif2apache(cf, r, res);
     return ret;
 #endif
@@ -468,29 +437,25 @@ process_zxid_simple_outcome:
  * In that case docgi() contains further zxid related steps to
  * pass the SSO attributes to the CGI environment. */
 
-/* Called by: */
+/* Called by:  handle_request */
 zxid_ses* zxid_mini_httpd_filter(zxid_conf* cf, const char* method, const char* uri_path, const char* qs, const char* cookie_hdr)
 {
   zxid_ses* ses;
   char buf[256];
-  D("===== START %s uri(%s) qs(%s) pid=%d gid=%d cwd(%s)", ZXID_REL, uri_path, STRNULLCHKNULL(qs), getpid(), getgid(), getcwd(buf,sizeof(buf)));
+  D(CC_GREENY("===== START %s uri(%s) qs(%s) uid=%d pid=%d gid=%d cwd(%s)"), ZXID_REL, uri_path, STRNULLCHKNULL(qs), getpid(), geteuid(), getegid(), getcwd(buf,sizeof(buf)));
   if (cf->wd && *cf->wd)
     chdir(cf->wd);
-  D_INDENT("minizx: ");
 
   zxid_is_wsp = 0;
-  if (zx_wildcard_pat_match(cf->wsp_pat, uri_path)) {
+  if (zx_match(cf->wsp_pat, uri_path)) {
     zxid_is_wsp = 1;
     ses = zxid_mini_httpd_wsp(cf, method, uri_path, qs);
-    D_DEDENT("minizx: ");
     return ses;
-  } else if (zx_wildcard_pat_match(cf->sso_pat, uri_path)) {
+  } else if (zx_match(cf->sso_pat, uri_path)) {
     ses = zxid_mini_httpd_sso(cf, method, uri_path, qs, cookie_hdr);
-    D_DEDENT("minizx: ");
     return ses;
   } else {
-    D("No SSO or WSP match(%s)",uri_path);
-    D_DEDENT("minizx: ");
+    D("No SSO or WSP match(%s) wsp_pat(%s) sso_pat(%s)", uri_path, STRNULLCHK(cf->wsp_pat), STRNULLCHK(cf->sso_pat));
     return 0;
   }
 }
