@@ -1,5 +1,5 @@
 /* mod_auth_saml.c  -  Handwritten functions for Apache mod_auth_saml module
- * Copyright (c) 2012-2013 Synergetics SA (sampo@synergetics.be), All Rights Reserved.
+ * Copyright (c) 2012-2013 Synergetics NV (sampo@synergetics.be), All Rights Reserved.
  * Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2008-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
@@ -17,6 +17,7 @@
  * 28.9.2012, changed zx_instance string to "mas", fixed parsing CGI for other page --Sampo
  * 13.2.2013, added WD option --Sampo
  * 21.6.2013, added SOAP WSP capability --Sampo
+ * 17.11.2013, move redir_to_content feature to zxid_simple() --Sampo
  *
  * To configure this module add to httpd.conf something like
  *
@@ -29,8 +30,6 @@
  *
  * http://httpd.apache.org/docs/2.2/developer/
  * http://modules.apache.org/doc/API.html
- *
- * Idea: set REMOTE_USER header
  */
 
 #define _LARGEFILE64_SOURCE   /* So off64_t is found, see: man 3 lseek64 */
@@ -43,6 +42,12 @@
 #include <zx/zxidutil.h>
 #include <zx/c/zxidvers.h>
 
+#ifdef MINGW
+/* apr.h defines these */
+#undef uid_t
+#undef gid_t
+#endif
+
 #include "ap_config.h"
 #include "ap_compat.h"
 #include "apr_strings.h"
@@ -90,13 +95,13 @@ static void chldinit(apr_pool_t* p, server_rec* s)
  * return:: Apache error code, typically OK, which allows Apache continue
  *     processing the request. */
 
-/* Called by:  chkuid */
+/* Called by:  chkuid x2 */
 static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
 {
   int ret = OK;
-  char* qs;
   char* name;
-  char* rs = 0;
+  //char* rs = 0;
+  //char* rs_qs;
   char* setcookie = 0;
   char* setptmcookie = 0;
   char* cookie = 0;
@@ -129,7 +134,7 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
 	apr_table_set(r->subprocess_env, name, av->map_val->s);
       }
     } else {
-      if ((zx_debug & ZX_DEBUG_MASK)>1)
+      if ((errmac_debug & ERRMAC_DEBUG_MASK)>1)
 	D("ATTR(%s)=VAL(%s)", at->name, STRNULLCHKNULL(at->val));
       else
 	D("ATTR(%s)=VAL(%.*s)", at->name, at->val?(int)MIN(35,strlen(at->val)):6, at->val?at->val:"(null)");
@@ -139,27 +144,26 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
       for (av = at->nv; av; av = av->n)
 	apr_table_set(r->subprocess_env, name, av->val);
     }
-    if (     !strcmp(at->name, "rs"))           rs = at->val;      /* Capture special */
-    else if (!strcmp(at->name, "idpnid"))       idpnid = at->val;
+    if      (!strcmp(at->name, "idpnid"))       idpnid = at->val;      /* Capture special */
     else if (!strcmp(at->name, "setcookie"))    setcookie = at->val;
     else if (!strcmp(at->name, "setptmcookie")) setptmcookie = at->val;
     else if (!strcmp(at->name, "cookie"))       cookie = at->val;
+    //else if (!strcmp(at->name, "rs"))         rs = at->val;
   }
+#if 0
   if (rs && rs[0] && rs[0] != '-') {
     /* N.B. RelayState was set by chkuid() "some other page" section by setting cgi.rs
      * to deflated and safe base64 encoded value which was then sent to IdP as RelayState.
      * It then came back from IdP and was decoded as one of the SSO attributes.
      * The decoding is controlled by <<tt: rsrc$rs$unsb64-inf$$ >>  rule in OUTMAP. */
-#if 1
     rs = zxid_unbase64_inflate(cf->ctx, -2, rs, 0);
     if (!rs) {
       ERR("Bad relaystate. Error in inflate. %d", 0);
       return HTTP_BAD_REQUEST;
     }
-#endif
-    qs = strchr(rs, '?');
-    if (qs
-	?(memcmp(r->uri, rs, qs-rs)||strcmp(r->args?r->args:"",qs+1))
+    rs_qs = strchr(rs, '?');
+    if (rs_qs
+	?(memcmp(r->uri, rs, rs_qs-rs)||strcmp(r->args?r->args:"",rs_qs+1))
 	:strcmp(r->uri, rs)) {  /* Different, need external or internal redirect */
       D("redirect(%s) redir_to_content=%d", rs, cf->redir_to_content);
       //r->uri = apr_pstrdup(r->pool, val);
@@ -172,6 +176,7 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
       }
     }
   }
+#endif
   
   if (setcookie && setcookie[0] && setcookie[0] != '-') {
     /* http://dev.ariel-networks.com/apr/apr-tutorial/html/apr-tutorial-19.html */
@@ -198,7 +203,7 @@ static int pool2apache(zxid_conf* cf, request_rec* r, struct zxid_attr* pool)
   
   //apr_table_setn(r->subprocess_env,
   //		 apr_psprintf(r->pool, "%sLDIF", cf->mod_saml_attr_prefix), ldif);
-  D("SSO OK ret(%d) uri(%s) filename(%s) path_info(%s) rs(%s)", ret, r->uri, r->filename, r->path_info, STRNULLCHKQ(rs));
+  D("SSO OK ret(%d) uri(%s) filename(%s) path_info(%s)", ret, r->uri, r->filename, r->path_info);
   return ret;
 }
 
@@ -231,7 +236,7 @@ static int send_res(zxid_conf* cf, request_rec* r, char* res)
   DD("CONTENT-LENGTH(%d)", len);
   ap_set_content_length(r, len);
   
-  if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("LEN(%d) strlen(%d) RES(%s)", len, (int)strlen(res), res);
+  if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("LEN(%d) strlen(%d) RES(%s)", len, (int)strlen(res), res);
   
   //register_timeout("send", r);
   ap_send_http_header(r);
@@ -245,7 +250,7 @@ static int send_res(zxid_conf* cf, request_rec* r, char* res)
  * This is considered internal function to mod_auth_saml, called by chkuid().
  * You should not call this directly, unless you know what you are doing. */
 
-/* Called by:  chkuid */
+/* Called by:  chkuid x2 */
 static char* read_post(zxid_conf* cf, request_rec* r)
 {
   int len, ret;
@@ -284,11 +289,11 @@ static char* read_post(zxid_conf* cf, request_rec* r)
     p += ret;
     len -= ret;
   }
-  if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("POST(%s)", res);
+  if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("POST(%s)", res);
   return res;
 }
 
-/* 0x6000 outf QS + JSON = no output
+/* 0x6000 outf QS + JSON = no output on successful sso, the attrubutes are in session
  * 0x1000 debug
  * 0x0e00 11 + 10 = Generate all HTML + Mgmt w/headers as string
  * 0x00a0 10 + 10 = Login w/headers as string + Meta w/headers as string
@@ -320,6 +325,8 @@ static int chkuid(request_rec* r)
   zxid_ses ses;
   ZERO(&cgi, sizeof(zxid_cgi));
   ZERO(&ses, sizeof(zxid_ses));
+  cgi.uri_path = r?r->uri:0;
+  cgi.qs = r?r->args:0;
 
   D("===== START %s req=%p uri(%s) args(%s) pid=%d cwd(%s)", ZXID_REL, r, r?STRNULLCHKNULL(r->uri):"(r null)", r?STRNULLCHKNULL(r->args):"(r null)", getpid(), getcwd(buf,sizeof(buf)));
   if (cf->wd && *cf->wd)
@@ -372,9 +379,9 @@ static int chkuid(request_rec* r)
 	strcpy(p, cf->redirect_hack_zxid_qs);
 	p[len] = '&';
 	strcpy(p+len+1, r->args);
-	r->args = p;
+	cgi.qs = r->args = p;
       } else {
-	r->args = cf->redirect_hack_zxid_qs;
+	cgi.qs = r->args = cf->redirect_hack_zxid_qs;
       }
       args_len = strlen(r->args);
     }
@@ -406,7 +413,7 @@ static int chkuid(request_rec* r)
     p = cf->url + url_len;
   
   if (url_len >= uri_len && !memcmp(p - uri_len, r->uri, uri_len)) {  /* Suffix match */
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("matched uri(%s) cf->url(%s) qs(%s) rs(%s) op(%c)", r->uri, cf->url, STRNULLCHKNULL(r->args), STRNULLCHKNULL(cgi.rs), cgi.op);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("matched uri(%s) cf->url(%s) qs(%s) rs(%s) op(%c)", r->uri, cf->url, STRNULLCHKNULL(r->args), STRNULLCHKNULL(cgi.rs), cgi.op);
     if (r->method_number == M_POST) {
       res = read_post(cf, r);   /* Will print some debug output */
       if (res) {
@@ -447,7 +454,7 @@ static int chkuid(request_rec* r)
 	goto process_zxid_simple_outcome;
     }
     /* not logged in, fall thru */
-  } else if (zx_wildcard_pat_match(cf->wsp_pat, r->uri)) {
+  } else if (zx_match(cf->wsp_pat, r->uri)) {
     /* WSP case */
     if (r->method_number == M_POST) {
       res = read_post(cf, r);   /* Will print some debug output */
@@ -471,7 +478,7 @@ static int chkuid(request_rec* r)
     }
   } else {
     /* Some other page. Just check for session. */
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("other page uri(%s) qs(%s) cf->url(%s) uri_len=%d url_len=%d", r->uri, STRNULLCHKNULL(r->args), cf->url, uri_len, url_len);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("other page uri(%s) qs(%s) cf->url(%s) uri_len=%d url_len=%d", r->uri, STRNULLCHKNULL(r->args), cf->url, uri_len, url_len);
     if (r->args && r->args[0] == 'l') {
       D("Detect login(%s)", r->args);
     } else
@@ -482,15 +489,7 @@ static int chkuid(request_rec* r)
       p[uri_len] = '?';
       strcpy(p+uri_len+1, r->args);
     }
-    DD("HERE3 args_len=%d cgi=%p k(%s) uri(%s) args(%s) rs(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), r->uri, STRNULLCHKNULL(r->args), p);
-    /* cgi.rs will be copied to ses->rs and from there in ab_pep to resource-id.
-     * We compress and safe_base64 encode it to protect any URL special characters.
-     * *** seems that at this point the p is not just rs, but the entire local URL --Sampo */
-    cgi.rs = zxid_deflate_safe_b64_raw(cf->ctx, -2, p);
-    if (cf->defaultqs && cf->defaultqs[0]) {
-      if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("DEFAULTQS(%s)", cf->defaultqs);
-      zxid_parse_cgi(cf, &cgi, apr_pstrdup(r->pool, cf->defaultqs));
-    }
+    D("HERE3 args_len=%d cgi=%p k(%s) uri(%s) args(%s) rs(%s)", args_len, &cgi, STRNULLCHKNULL(cgi.skin), r->uri, STRNULLCHKNULL(r->args), p);
     if (cgi.sid && cgi.sid[0] && zxid_get_ses(cf, &ses, cgi.sid)) {
       res = zxid_simple_ses_active_cf(cf, &cgi, &ses, 0, AUTO_FLAGS);
       if (res)
@@ -505,18 +504,18 @@ step_up:
 
 process_zxid_simple_outcome:
   if (cookie_hdr && cookie_hdr[0]) {
-    D("Passing cookie(%s) to environment", cookie_hdr);
+    D("Passing previous cookie(%s) to environment", cookie_hdr);
     zxid_add_attr_to_ses(cf, &ses, "cookie", zx_dup_str(cf->ctx, cookie_hdr));
   }
 
   switch (res[0]) {
   case 'L':
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("REDIR(%s)", res);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("REDIR(%s)", res);
     apr_table_setn(r->headers_out, "Location", res+10);
     D_DEDENT("chkuid: ");
     return HTTP_SEE_OTHER;
   case 'C':
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("CONTENT(%s)", res);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("CONTENT(%s)", res);
     ret = send_res(cf, r, res);
     D_DEDENT("chkuid: ");
     return ret;
@@ -531,7 +530,7 @@ process_zxid_simple_outcome:
     return ret;
 #if 0
   case 'd': /* Logged in case */
-    if (zx_debug & MOD_AUTH_SAML_INOUT) INFO("SSO OK LDIF(%s)", res);
+    if (errmac_debug & MOD_AUTH_SAML_INOUT) INFO("SSO OK LDIF(%s)", res);
     D("SSO OK pre uri(%s) filename(%s) path_info(%s)", r->uri, r->filename, r->path_info);
     ret = ldif2apache(cf, r, res);
     D_DEDENT("chkuid: ");
@@ -557,9 +556,9 @@ process_zxid_simple_outcome:
 /* Called by: */
 static const char* set_debug(cmd_parms* cmd, void* st, const char* arg) {
   char buf[256];
-  D("old debug=%x, new debug(%s)", zx_debug, arg);
-  sscanf(arg, "%i", &zx_debug);
-  INFO("debug=0x%x now arg(%s) cwd(%s)", zx_debug, arg, getcwd(buf, sizeof(buf)));
+  D("old debug=%x, new debug(%s)", errmac_debug, arg);
+  sscanf(arg, "%i", &errmac_debug);
+  INFO("debug=0x%x now arg(%s) cwd(%s)", errmac_debug, arg, getcwd(buf, sizeof(buf)));
   return 0;
 }
 
@@ -603,7 +602,7 @@ const command_rec zxid_apache_commands[] = {
 static void* dirconf(apr_pool_t* p, char* d)
 {
   zxid_conf* cf;
-  strncpy(zx_instance, "\tmas", sizeof(zx_instance));
+  strncpy(errmac_instance, "\tmas", sizeof(errmac_instance));
   cf = apr_palloc(p, sizeof(zxid_conf));
   ZERO(cf, sizeof(zxid_conf));
   cf->ctx = apr_palloc(p, sizeof(struct zx_ctx));
