@@ -60,7 +60,7 @@ extern int verbose;  /* defined in option parsing in zxbusd.c */
 extern zxid_conf* zxbus_cf;
 
 #if 0
-/* Called by:  stomp_got_err */
+/* Called by: */
 static struct hi_pdu* stomp_encode_start(struct hi_thr* hit)
 {
   struct hi_pdu* resp = hi_pdu_alloc(hit,"stomp_enc_start");
@@ -71,7 +71,7 @@ static struct hi_pdu* stomp_encode_start(struct hi_thr* hit)
 
 /*() Send ERROR to remote client. */
 
-/* Called by:  stomp_decode x2, stomp_got_login x2, zxbus_persist x2 */
+/* Called by:  stomp_frame_err, stomp_got_login x2, zxbus_persist x2 */
 int stomp_err(struct hi_thr* hit, struct hi_io* io, struct hi_pdu* req, const char* ecode, const char* emsg)
 {
   int len;
@@ -89,6 +89,7 @@ int stomp_err(struct hi_thr* hit, struct hi_io* io, struct hi_pdu* req, const ch
 
 /*() Send an error early on in decode process */
 
+/* Called by:  stomp_decode x2 */
 static int stomp_frame_err(struct hi_thr* hit, struct hi_io* io, struct hi_pdu* req, const char* emsg)
 {
   /* At this early stage the req is still a io->cur_pdu. We need to
@@ -295,7 +296,7 @@ static void stomp_got_ack(struct hi_thr* hit, struct hi_io* io, struct hi_pdu* r
   parent = resp->parent;
   ASSERT(parent);
   
-  if (zx_debug>1)
+  if (errmac_debug>1)
     D("ACK par_%p->len=%d rq_%p->len=%d\nparent->body(%.*s)\n   req->body(%.*s)", parent, parent->ad.delivb.len, resp->req, resp->req->ad.stomp.len, parent->ad.delivb.len, parent->ad.delivb.body, resp->req->ad.stomp.len, resp->req->ad.stomp.body);
   else
     D("ACK par_%p->len=%d rq_%p->len=%d", parent, parent->ad.delivb.len, resp->req, resp->req->ad.stomp.len);
