@@ -209,6 +209,7 @@
 
 /* control point enumerators */
 
+#define TAS3_WSC_RQ_OUT   "urn:tas3:ctlpt:wsc:rq:out"
 #define TAS3_PEP_RQ_OUT   "urn:tas3:ctlpt:pep:rq:out"
 #define TAS3_PEP_RQ_IN    "urn:tas3:ctlpt:pep:rq:in"
 #define TAS3_PEP_RS_PARSE "urn:tas3:ctlpt:pep:rs:parse"
@@ -224,10 +225,11 @@
 #define TAS3_STATUS_NOSIG   "urn:tas3:status:nosig"
 #define TAS3_STATUS_BADSIG  "urn:tas3:status:badsig"
 #define TAS3_STATUS_BADCOND "urn:tas3:status:badcond"
-#define TAS3_STATUS_EXPIRED "urn:tas3:status:expired" /* credential expired */
 #define TAS3_STATUS_REPLAY  "urn:tas3:status:replay"
 #define TAS3_STATUS_BADXML  "urn:tas3:status:badxml"
 #define TAS3_STATUS_FAULT   "urn:tas3:status:fault"
+#define TAS3_STATUS_EXPIRED "urn:tas3:status:expired" /* credential expired */
+#define TAS3_STATUS_EPR_NOT_FOUND "urn:tas3:status:epr-not-found"
 
 /* Special value for ObligationId to identify the SOL1 processing engine. */
 
