@@ -17,6 +17,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/stat.h>  /* for mkdir(2) */
 
 #include "errmac.h"
 #include "zx.h"
@@ -30,31 +31,33 @@
 
 char* help =
 "zxcot  -  Circle-of-Trust and metadata management tool R" ZXID_REL "\n\
-Copyright (c) 2012 Synergetics SA (sampo@synergetics.be), All Rights Reserved.\n\
+Copyright (c) 2012-2013 Synergetics SA (sampo@synergetics.be), All Rights Reserved.\n\
 Copyright (c) 2009-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.\n\
 NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
 See http://www.apache.org/licenses/LICENSE-2.0\n\
 Send well researched bug reports to the author. Home: http://zxid.org\n\
 \n\
-Usage: zxcot [options] [dir]         # Gives listing of metadata\n\
-       zxcot -a [options] [dir] <meta.xml\n\
-       zxcot -b [options] [dir] <epr.xml\n\
-       curl https://site.com/metadata.xml | zxcot -a [options] [dir]\n\
-       zxcot -g https://site.com/metadata.xml [options] [dir]\n\
-       zxcot -m [options] >meta.xml  # Generate our own metadata\n\
+Usage: zxcot [options] [cotdir]         # Gives listing of metadata\n\
+       zxcot -c CPATH=/var/zxid/ -dirs  # Creates directory hierarchy\n\
+       zxcot -a [options] [cotdir] <meta.xml  # Import metadata\n\
+       zxcot -b [options] [dimddir] <epr.xml  # Register EPR\n\
+       curl https://site.com/metadata.xml | zxcot -a [options] [cotdir]\n\
+       zxcot -g https://site.com/metadata.xml [options] [cotdir]\n\
+       zxcot -m [options] >meta.xml     # Generate our own metadata\n\
        zxcot -p https://site.com/metadata.xml\n\
   [dir]            CoT directory. Default /var/zxid/cot\n\
-  -c CONF          Optional configuration string (default -c PATH=/var/zxid/)\n\
+  -c CONF          Optional configuration string (default -c CPATH=/var/zxid/)\n\
                    Most of the configuration is read from " ZXID_CONF_PATH "\n\
                    N.B. If VURL and/or VPATH are used, you should set\n\
                    environment variables that affect virtualization, e.g.\n\
                      HTTP_HOST=example.com:8443 SERVER_PORT=8443 SCRIPT_NAME=zxidhlo zxcot -m\n\
   -ci              IdP conf, synonym for -c IDP_ENA=1\n\
-  -a               Add metadata from stdin\n\
+  -dirs            Create configuration directory hierarchy\n\
+  -a               Add (someone else's) metadata from stdin\n\
   -b               Register Web Service, add Service EPR from stdin\n\
   -bs              Register Web Service and Bootstrap, add Service EPR from stdin\n\
   -e endpoint abstract entid servicetype   Construct and dump EPR to stdout.\n\
-  -g URL           Do HTTP(S) GET to URL and add as metadata (if compiled w/libcurl)\n\
+  -g URL           Do HTTP(S) GET to URL (aka WKL) and add as metadata (if compiled w/libcurl)\n\
   -n               Dryrun. Do not actually add the metadata. Instead print it to stdout.\n\
   -s               Swap columns, for easier sorting by URL\n\
   -m               Output metadata of this installation (our own metadata). Caveat: If your\n\
@@ -89,6 +92,8 @@ char* dimddir;
 char* uiddir;
 zxid_conf* cf = 0;
 
+static void zxmkdirs();
+
 /* Called by:  main x8, zxbusd_main, zxbuslist_main, zxbustailf_main, zxcall_main, zxcot_main, zxdecode_main */
 static void opt(int* argc, char*** argv, char*** env)
 {
@@ -181,13 +186,17 @@ static void opt(int* argc, char*** argv, char*** env)
     case 'd':
       switch ((*argv)[0][2]) {
       case '\0':
-	++zx_debug;
+	++errmac_debug;
 	continue;
       case 'c':
 	ss = zxid_show_conf(cf);
 	fprintf(stderr, "\n======== CONF ========\n%.*s\n^^^^^^^^ CONF ^^^^^^^^\n",ss->len,ss->s);
 	continue;
       }
+      if (!strcmp((*argv)[0],"-dirs")) {
+	zxmkdirs();
+	exit(0);
+      }
       break;
 
     case 's':
@@ -297,6 +306,67 @@ path_to_dir:
   }
 }
 
+/* --------- Make zxid config directories --------- */
+
+static const char* mkdirs_list[] = {
+"ses",
+"user",
+"uid",
+"nid",
+"log",
+"log/rely",
+"log/issue",
+"pem",
+"cot",
+"inv",
+"dimd",
+"uid/.all",
+"uid/.all/.bs",
+"tmp",
+"ch",
+"ch/default",
+"ch/default/.ack",
+"ch/default/.del"
+};
+
+/* Called by:  opt */
+static void zxmkdirs()
+{
+  char path[ZXID_MAX_BUF];
+  char* p;
+  const char** dir;
+  int len;
+#define ZXID_PATH_LENGTH_MARGIN (sizeof("ch/default/.del")+10) /* Accommodate longest subdir */
+  len = snprintf(path, sizeof(path)-ZXID_PATH_LENGTH_MARGIN, "%s", cf->path);
+  if (len > sizeof(path)-ZXID_PATH_LENGTH_MARGIN) {
+    ERR("CPATH %s too long. len=%d, space=%d", cf->path, len, (int)(sizeof(path)-ZXID_PATH_LENGTH_MARGIN));
+    exit(1);
+  }
+
+  for (p = path+len-1; p > path && *p != '/'; --p) ;  /* Handle CPATH=/var/zxid/idp */
+
+  if (MKDIR(path, 02770) < 0) {
+    if (errno == EEXIST) {
+      INFO("Directory %s already exists. Will still try to create hierarchy under it.", path);
+    } else {
+      ERR("Failed to create directory hierarchy at %s: %d (%s) (perhaps nonexistent parent directory or permissions problem)", path, errno, STRERROR(errno));
+      exit(1);
+    }
+  } else {
+    D("Created %s", path);
+  }
+  
+  for (dir = mkdirs_list; *dir; ++dir) {
+    len = snprintf(path, sizeof(path)-ZXID_PATH_LENGTH_MARGIN, "%s%s", cf->path, *dir);
+    if (MKDIR(path, 02770) < 0) {
+      ERR("Failed to create directory %s: %d (%s)", path, errno, STRERROR(errno));
+    } else {
+      D("Created %s", path);
+    }
+  }
+  INFO("Created directories. You should inspect their ownership and permissions to ensure the webserver can read and write them, yet outsiders can not access them. You may want to run  chown -R www-data %s", cf->path);
+}
+
 /* --------------- reg_svc --------------- */
 
 /*() IdP and Discovery. Register service metadata to /var/zxid/idpdimd/XX,
@@ -313,13 +383,14 @@ static int zxid_reg_svc(zxid_conf* cf, int bs_reg, int dry_run, const char* ddim
 {
   char sha1_name[28];
   char path[ZXID_MAX_BUF];
-  int got, fd;
+  int got;
+  fdtype fd;
   struct zx_root_s* r;
   zxid_epr* epr;
   struct zx_str* ss;
   struct zx_str* tt;
   
-  read_all_fd(fileno(stdin), buf, sizeof(buf)-1, &got);  /* Read EPR */
+  read_all_fd(fdstdin, buf, sizeof(buf)-1, &got);  /* Read EPR */
   buf[got] = 0;
   
   r = zx_dec_zx_root(cf->ctx, got, buf, "cot reg_svc");
@@ -422,7 +493,8 @@ static int zxid_reg_svc(zxid_conf* cf, int bs_reg, int dry_run, const char* ddim
 /* Called by:  zxcot_main */
 static int zxid_addmd(zxid_conf* cf, char* mdurl, int dry_run, const char* dcot)
 {
-  int got, fd;
+  int got;
+  fdtype fd;
   char* p;
   zxid_entity* ent;
   struct zx_str* ss;
@@ -430,7 +502,7 @@ static int zxid_addmd(zxid_conf* cf, char* mdurl, int dry_run, const char* dcot)
   if (mdurl) {
     ent = zxid_get_meta(cf, mdurl);
   } else {
-    read_all_fd(fileno(stdin), buf, sizeof(buf)-1, &got);
+    read_all_fd(fdstdin, buf, sizeof(buf)-1, &got);
     buf[got] = 0;
     p = buf;
     ent = zxid_parse_meta(cf, &p, buf+got);
@@ -497,7 +569,7 @@ static int zxid_lscot_line(zxid_conf* cf, int col_swap, const char* dcot, const
   char* p;
   int got = read_all(ZXID_MAX_MD, buf, "zxcot line", 1, "%s%s", dcot, den);
   if (!got) {
-    ERR("Zero data in file(%s%s). If cot directory does not exist consider running zxmkdirs.sh", dcot, den);
+    ERR("Zero data in file(%s%s). If cot directory does not exist consider running zxcot -dirs", dcot, den);
     return 1;
   }
   p = buf;
@@ -566,7 +638,7 @@ extern int zxid_suppress_vpath_warning;
 /* Called by: */
 int zxcot_main(int argc, char** argv, char** env)
 {
-  strncpy(zx_instance, "cot", sizeof(zx_instance));
+  strncpy(errmac_instance, "cot", sizeof(errmac_instance));
   zxid_suppress_vpath_warning = 1;
   cf = zxid_new_conf_to_cf(0);
 
