@@ -125,7 +125,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
 	if (conf_path)
 	  read_all(sizeof(buf), buf, "new conf path in opt", 1, "%s", conf_path);
 	else
-	  read_all(sizeof(buf), buf, "no conf path in opt", 1, "%szxid.conf", cf->path);
+	  read_all(sizeof(buf), buf, "no conf path in opt", 1, "%s" ZXID_CONF_FILE, cf->path);
 	zxid_parse_conf(cf, buf);
 	conf_path = (char*)1;
       }
@@ -135,12 +135,12 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
     case 'd':
       switch ((*argv)[0][2]) {
       case '\0':
-	++zx_debug;
+	++errmac_debug;
 	continue;
       case 'i':  if ((*argv)[0][3]) break;
 	++(*argv); --(*argc);
 	if (!(*argc)) break;
-	strcpy(zx_instance, (*argv)[0]);
+	strcpy(errmac_instance, (*argv)[0]);
 	continue;
       }
       break;
@@ -189,6 +189,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
       }
       break;
 #endif
+#ifndef MINGW
     case 'k':
       switch ((*argv)[0][2]) {
       case '\0':
@@ -199,6 +200,7 @@ void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
 	continue;
       }
       break;
+#endif
 
     case 'l':
       switch ((*argv)[0][2]) {
@@ -426,7 +428,7 @@ int main(int argc, char** argv, char** env)
   if (got != 2)
     exit(2);
   fprintf(stderr, "=================== Running ===================\n");
-  ++zx_debug;
+  ++errmac_debug;
   zxid_set_opt(cf, 6, 0);
 #endif
   cf->nosig_fatal = 0;  // *** For SimpleSign the signature is checked at other level
@@ -468,7 +470,7 @@ int main(int argc, char** argv, char** env)
       cont_len = getenv("CONTENT_LENGTH");
       if (cont_len) {
 	sscanf(cont_len, "%d", &got);
-	if (read_all_fd(fileno(stdin), buf, got, &got) == -1) {
+	if (read_all_fd(fdstdin, buf, got, &got) == -1) {
 	  perror("Trouble reading post content");
 	} else {
 	  buf[got] = 0;
@@ -517,8 +519,10 @@ int main(int argc, char** argv, char** env)
       return 0;
     break;
   case 'L':
-    if (zxid_start_sso(cf, &cgi))
+    if (ss = zxid_start_sso_location(cf, cgi)) {
+      printf("%.*s", ss->len, ss->s);
       return 0;
+    }
     break;
   case 'A':
     D("Process artifact(%s)", cgi.saml_art);
@@ -545,7 +549,7 @@ int main(int argc, char** argv, char** env)
     }
     break;
   case 'B':  /* Metadata */
-    write_all_fd(fileno(stdout), "Content-Type: text/xml\r\n\r\n", sizeof("Content-Type: text/xml\r\n\r\n")-1);
+    write_all_fd(fdstdout, "Content-Type: text/xml\r\n\r\n", sizeof("Content-Type: text/xml\r\n\r\n")-1);
     return zxid_send_sp_meta(cf, &cgi);
   default: D("unknown op(%c)", cgi.op);
   }
