@@ -206,11 +206,14 @@ static int zx_chk_el_ord(struct zx_elem_s* x)
     if (ed->el_order[j] == ZX_TOK_NOT_FOUND) {
       if (x->g.tok == ZX_TOK_NOT_FOUND || !IN_RANGE(x->g.tok & ZX_TOK_TOK_MASK, 0, zx__ELEM_MAX)) {
 	ef = zx_el_tab + MINMAX(ed->tok & ZX_TOK_TOK_MASK, 0, zx__ELEM_MAX);
-	INFO("Unknown <%.*s> token(0x%06x) as %d. child of <%s> 0x%06x (%d,%d)", x->g.len, x->g.s, x->g.tok, n, ef->name, ed->tok, i, j);
+	/* *** ideally this should be flagged as error, but problem is that we
+	 *     permit freeform bodies so there are a lot of unknown tokens like this. */
+	D("Unknown <%.*s> token(0x%06x) as %d. child of <%s> 0x%06x (%d,%d)", x->g.len, x->g.s, x->g.tok, n, ef->name, ed->tok, i, j);
       } else {
 	et = zx_el_tab + (x->g.tok & ZX_TOK_TOK_MASK);
 	ef = zx_el_tab + MINMAX(ed->tok & ZX_TOK_TOK_MASK, 0, zx__ELEM_MAX);
 	ERR("Known <%s> tok(0x%06x) in wrong place as %d. child of <%s> tok(0x%06x) (%d,%d)", et->name, x->g.tok, n, ef->name, ed->tok, i, j);
+	// *** we should really dump the whole message into log
       }
       return n;
     }
@@ -308,10 +311,12 @@ struct zx_attr_s* zx_ord_ins_at(struct zx_elem_s* x, struct zx_attr_s* in_at)
  * Thus we need to reverse them to get them to right order. We
  * take this opportunity to also check that the ordering is correct
  * and also to sort the XML attributes.
- * N.B. This function is not recursive: only one level is reveresed.
- * Called from dec-templ.c for CSE elimination. */
+ * Called from dec-templ.c for CSE elimination.
+ * N.B. This function is not recursive: only one level is reversed.
+ * N.B2. Although decoder returns lists in reverse order, we try
+ * to maintain as common representation the forward ordered list. */
 
-/* Called by:  zx_DEC_elem, zxenc_pubkey_enc, zxenc_symkey_enc, zxid_ac_desc, zxid_add_fed_tok2epr, zxid_ar_desc, zxid_az_soap, zxid_contact_desc, zxid_idp_sso_desc, zxid_key_desc, zxid_key_info, zxid_mk_a7n, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_di_req_svc, zxid_mk_mni, zxid_mk_saml_resp, zxid_mk_xac_az, zxid_mk_xacml_resp, zxid_mk_xacml_simple_at, zxid_mni_desc, zxid_nimap_desc, zxid_org_desc, zxid_slo_desc, zxid_sp_meta, zxid_sp_sso_desc, zxid_sso_desc, zxid_wsc_prep, zxid_wsf_decor x2, zxid_wsp_decorate, zxsig_sign x3 */
+/* Called by:  zx_DEC_elem, zxenc_pubkey_enc, zxenc_symkey_enc, zxid_ac_desc, zxid_add_env_if_needed, zxid_add_fed_tok2epr, zxid_ar_desc, zxid_az_soap, zxid_contact_desc, zxid_idp_sso_desc, zxid_key_desc, zxid_key_info, zxid_mk_a7n, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_di_req_svc, zxid_mk_mni, zxid_mk_saml_resp, zxid_mk_xac_az, zxid_mk_xacml_resp, zxid_mk_xacml_simple_at, zxid_mni_desc, zxid_nimap_desc, zxid_org_desc, zxid_slo_desc, zxid_sp_meta, zxid_sp_sso_desc, zxid_sso_desc, zxid_wsc_prep, zxid_wsf_decor x2, zxid_wsp_decorate, zxsig_sign x3 */
 void zx_reverse_elem_lists(struct zx_elem_s* x)
 {
   struct zx_elem_s* iternode;
@@ -506,7 +511,9 @@ static struct zx_elem_s* zx_el_lookup(struct zx_ctx* c, struct zx_elem_s* x, str
       goto unknown_el;
   } else {
 unknown_el:
-    INFO("Unknown element <%.*s>, child of <%.*s>", ((int)(c->p - full_name)), full_name, x->g.len, x->g.s);
+    // Unknown element warnings are quite frequent and just clutter the logs. Downgrade.
+    //INFO("Unknown element <%.*s>, child of <%.*s>", ((int)(c->p - full_name)), full_name, x->g.len, x->g.s);
+    D("Unknown element <%.*s>, child of <%.*s>", ((int)(c->p - full_name)), full_name, x->g.len, x->g.s);
     el = ZX_ZALLOC(c, struct zx_elem_s);
     tok = ZX_TOK_NOT_FOUND;
   }
@@ -624,7 +631,9 @@ void zx_prepare_dec_ctx(struct zx_ctx* c, struct zx_ns_s* ns_tab, int n_ns, cons
   c->lim = lim;
 }
 
-/*(i) Decode arbitary xml with zx_ns_tab set of namespaces and parsers. */
+/*(i) Decode arbitary xml with zx_ns_tab set of namespaces and parsers.
+ * The resulting data structure has linked lists in *inverted* order,
+ * i.e. last tag is first element of the list. */
 
 /* Called by:  main x6, sig_validate, test_ibm_cert_problem, zxid_add_env_if_needed x2, zxid_dec_a7n, zxid_decode_redir_or_post, zxid_decrypt_nameid, zxid_decrypt_newnym, zxid_di_query, zxid_find_epr, zxid_gen_boots, zxid_get_ses_sso_a7n x2, zxid_idp_soap_parse, zxid_mk_sa_attribute_ss, zxid_mk_xacml_simple_at, zxid_parse_meta, zxid_print_session, zxid_reg_svc, zxid_soap_call_raw, zxid_sp_soap_parse, zxid_str2a7n, zxid_str2nid, zxid_str2token, zxid_wsp_validate */
 struct zx_root_s* zx_dec_zx_root(struct zx_ctx* c, int len, const char* start, const char* func)
